#ifndef SWR_POLYGON_HPP_INCLUDED
#define SWR_POLYGON_HPP_INCLUDED


#include"swr_point.hpp"
#include"swr_vector.hpp"
#include"swr_color.hpp"
#include"swr_sincos.hpp"
#include<cstdint>




namespace swr{




struct
SortedPolygon
{
  Color  color;

  Point  highest;
  Point   middle;
  Point   lowest;

  constexpr SortedPolygon(const Color&  color_, const Point&  h, const Point&  m, const Point&  l):
  color(color_),
  highest(h),
  middle(m),
  lowest(l){}


  bool  test() const
  {
    return((highest.y >= middle.y) &&
           (highest.y >= lowest.y));
  }

};




struct
Polygon
{
  Color  color;

  Point  a_point;
  Point  b_point;
  Point  c_point;

  Polygon(){}

  constexpr Polygon(const Color&  color_, const Point&  a, const Point&  b, const Point&  c):
  color(color_),
  a_point(a),
  b_point(b),
  c_point(c){}


  Polygon&  operator+=(const Point&  offset);
  Polygon&  operator-=(const Point&  offset);

  void  modify(const Point&  offset, const RotationParameter&  rotparam);

  SortedPolygon  to_sorted() const;

  Vector  get_vector() const;

  void  print() const;

};




}



#endif




