#include"swr_viewer.hpp"
#include"swr_linecontext.hpp"
#include<cstdio>
#include<cmath>




namespace swr{


constexpr double  pi = 3.141592653589793;


constexpr
int
to_degree(double  radian)
{
  return radian*180.0/pi;
}


void
Viewer::
reset(const World&  world, bool  lighting)
{
  transformated_lines.clear();
  transformated_polygons.clear();

    for(auto&  obj: world.objects)
    {
        for(auto&  poly: obj.polygons)
        {
          auto  tmp = poly;

          obj.rotate(tmp.a_point);
          obj.rotate(tmp.b_point);
          obj.rotate(tmp.c_point);

            if(lighting)
            {
              auto  v = tmp.get_vector();

              v.normalize();

              auto  d = Vector::dot_product(v,world.light);

              auto  degree = std::abs(to_degree(std::cos(d)));

                if(degree <= 90)
                {
                  constexpr int  unit = (255<<16)/90;

                  degree = 90-degree;

                  tmp.color.r = ((unit*degree)/255*tmp.color.r)>>16;
                  tmp.color.g = ((unit*degree)/255*tmp.color.g)>>16;
                  tmp.color.b = ((unit*degree)/255*tmp.color.b)>>16;
                }

              else
                {
                  tmp.color.r = 0;
                  tmp.color.g = 0;
                  tmp.color.b = 0;
                }
            }


          append(tmp);
        }


        for(auto&  poly: obj.lines)
        {
          auto  tmp = poly;

          obj.rotate(tmp.a);
          obj.rotate(tmp.b);

          transformated_lines.emplace_back(tmp);
        }
    }


    for(auto&  poly: world.polygons)
    {
      append(poly);
    }


    for(auto&  ln: world.lines)
    {
      auto  tmp = ln;

      tmp.a -= center;
      tmp.b -= center;

      rotate(tmp.a);
      rotate(tmp.b);

      transformated_lines.emplace_back(tmp);
    }
}


void
Viewer::
append(const Polygon&  poly)
{
  auto  tmp = poly;

  tmp.a_point -= center;
  tmp.b_point -= center;
  tmp.c_point -= center;

  rotate(tmp.a_point);
  rotate(tmp.b_point);
  rotate(tmp.c_point);

  transformated_polygons.emplace_back(tmp.to_sorted());
}


void
Viewer::
render()
{
  screen.clear();

    for(auto&  poly: transformated_polygons)
    {
      PolygonRenderer(poly,screen);
    }


    for(auto&  ln: transformated_lines)
    {
      ln.render(screen);
    }
}


}




