#include<SDL.h>
#include<limits>
#include<algorithm>
#include"swr_viewer.hpp"



using namespace swr;


namespace{


constexpr int  screen_w = 480;
constexpr int  screen_h = 480;


SDL_Window*
window;


SDL_Renderer*
rend;


SDL_Texture*
texture;


SDL_PixelFormat*
format;


int  looping;


World
world;


Viewer
viewer;




template<typename  T>
void
put(uint8_t*&  ptr, uint32_t  color)
{
  *reinterpret_cast<T*>(ptr) = color;

  ptr += sizeof(T);
}


void
update()
{
  int  pitch;

  void*  ptr;

  viewer.reset(world,true);
  viewer.render();

  SDL_LockTexture(texture,nullptr,&ptr,&pitch);

  auto  p_base = (uint8_t*)ptr;

    for(int  y = viewer.screen.get_height()-1;  y >= 0;  --y)
    {
      auto  p = p_base         ;
                p_base += pitch;

        for(int  x = 0;  x < viewer.screen.get_width();  ++x)
        {
          auto&  e = viewer.screen.get_element(x,y);

          auto  color = SDL_MapRGB(format,e.color.r,e.color.g,e.color.b);

            switch(format->BytesPerPixel)
            {
              case(1): put<uint8_t >(p,color);break;
              case(2): put<uint16_t>(p,color);break;
              case(4): put<uint32_t>(p,color);break;
              default:
                report;
            }
        }
    }


  SDL_UnlockTexture(texture);

  SDL_RenderCopy(rend,texture,nullptr,nullptr);

  SDL_RenderPresent(rend);
}


void
keydown(const SDL_KeyboardEvent&  evt)
{
  auto&  obj = world.objects.back();

    switch(evt.keysym.sym)
    {
      case(SDLK_LEFT ): obj.y_degree-=4;break;
      case(SDLK_RIGHT): obj.y_degree+=4;break;
      case(SDLK_UP   ): obj.x_degree+=4;break;
      case(SDLK_DOWN ): obj.x_degree-=4;break;
    }


  update();
}


void
make_trigonal_pyramid(const Point&  base, int  x_width, int  y_width, int  z_width)
{
  world.objects.emplace_back(Object());

  Object&  o = world.objects.back();

  o.push_polygon(Color(255,0,0),Point(base.x        ,base.y        ,base.z),
                                Point(base.x        ,base.y+y_width,base.z),
                                Point(base.x+x_width,base.y        ,base.z));
  o.push_polygon(Color(0,255,0),Point(base.x,base.y        ,base.z        ),
                                Point(base.x,base.y+y_width,base.z        ),
                                Point(base.x,base.y        ,base.z+z_width));
  o.push_polygon(Color(0,0,255),Point(base.x,base.y+y_width,base.z        ),
                                Point(base.x+x_width,base.y,base.z        ),
                                Point(base.x        ,base.y,base.z+z_width));
  o.push_polygon(Color(255,255,0),Point(base.x        ,base.y,base.z        ),
                                Point(base.x+x_width,base.y,base.z        ),
                                Point(base.x        ,base.y,base.z+z_width));
}


}




int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);

  atexit(SDL_Quit);

  window = SDL_CreateWindow("swrtest - " __DATE__,32,8,screen_w,screen_h,0);

  rend = SDL_CreateRenderer(window,-1,SDL_RENDERER_SOFTWARE);

  SDL_RendererInfo  info;

  SDL_GetRendererInfo(rend,&info);

  auto  pixelformatcode = info.texture_formats[0];

  format = SDL_AllocFormat(pixelformatcode);

  texture = SDL_CreateTexture(rend,pixelformatcode,SDL_TEXTUREACCESS_STREAMING,screen_w,screen_h);

  looping = true;

  viewer.screen.resize(screen_w,screen_h);


  viewer.center.z +=300;

  constexpr int  v = 200;

  make_trigonal_pyramid(Point(0,0,0),v,v,v);

  world.light.x =1;
  world.light.y =-1;
  world.light.z=-1;

  update();


  static SDL_Event  evt;

    while(looping)
    {
        while(SDL_PollEvent(&evt))
        {
            switch(evt.type)
            {
              case(SDL_KEYDOWN): keydown(evt.key);break;
              case(SDL_QUIT):        looping = false;                break;
            }
        }


      SDL_Delay(20);
    }


  SDL_FreeFormat(format);

  SDL_DestroyTexture(texture);
  SDL_DestroyRenderer(rend);
  SDL_DestroyWindow(window);

  return 0;
}




